@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
- (C) 2008,2009,2010 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+ (C) 2008-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 from PyQt4.QtCore import QPointF, QRectF, QVariant
@@ -214,7 +214,7 @@ class HandBoard(Board):
         tile = event.mimeData().tile or event.mimeData().meld[0]
         localY = self.mapFromScene(QPointF(event.scenePos())).y()
         centerY = self.rect().height()/2.0
-        newLowerHalf =  localY >= centerY
+        newLowerHalf = localY >= centerY
         noMansLand = centerY / 6
         if -noMansLand < localY - centerY < noMansLand and not tile.isBonus():
             doAccept = False
@@ -251,7 +251,7 @@ class HandBoard(Board):
         return result
 
     def receive(self, tile=None, meld=None):
-        """receive a tile  or meld and return the meld this tile becomes part of"""
+        """receive a tile or meld and return the meld this tile becomes part of"""
         if tile:
             if tile.isBonus():
                 if tile.board == self:
@@ -274,6 +274,10 @@ class HandBoard(Board):
             self.player.addMeld(meld)
             self.sync(adding=meld.tiles)
             senderBoard.remove(meld=meld)
+        meld.tiles = sorted(meld.tiles, key=lambda x: x.xoffset)
+        if any(x.focusable for x in meld.tiles):
+            for idx, tile in enumerate(meld.tiles):
+                tile.focusable = idx == 0
         return meld
 
     def lowerHalfTiles(self):
@@ -281,7 +285,8 @@ class HandBoard(Board):
         return list(x for x in self.tiles if x.yoffset > 0)
 
     def newTilePositions(self):
-        """returns list(TileAttr). The tiles are not associated to any board."""
+        """returns list(TileAttr) for all tiles except bonus tiles.
+        The tiles are not associated to any board."""
         result = list()
         isScoringGame = self.player.game.isScoringGame()
         newUpperMelds = self.player.exposedMelds[:]
@@ -318,11 +323,11 @@ class HandBoard(Board):
                     result.append(newTile)
                     meldX += 1
                 meldX += meldDistance
-        self.newBonusPositions(result)
         return sorted(result, key=lambda x: x.yoffset * 100 + x.xoffset)
 
     def newBonusPositions(self, newTilePositions):
-        """calculate places for bonus tiles. Put them all in one row,
+        """returns list(TileAttr)
+        calculate places for bonus tiles. Put them all in one row,
         right adjusted. If necessary, extend to the right even outside of our board"""
         positions = list(x.xoffset for x in newTilePositions if x.yoffset==0)
         upperLen = max(positions) if positions else 0
@@ -338,12 +343,14 @@ class HandBoard(Board):
         newBonusTiles = list(TileAttr(x) for x in self.player.bonusTiles)
         xPos = 13 - len(newBonusTiles)
         xPos = max(xPos, tileLen)
+        result = list()
         for bonus in sorted(newBonusTiles, key=tileKey):
-            bonus.xoffset,  bonus.yoffset = xPos,  bonusY
+            bonus.xoffset, bonus.yoffset = xPos, bonusY
             bonus.focusable = self.player.game.isScoringGame()
             bonus.dark = False
-            newTilePositions.append(bonus)
+            result.append(bonus)
             xPos += 1
+        return result
 
     def calcPlaces(self, adding=None):
         """returns a dict. Keys are existing tiles, Values are TileAttr instances.
@@ -352,6 +359,8 @@ class HandBoard(Board):
         allTiles = self.tiles[:]
         if adding:
             allTiles.extend(adding)
+        # process bonus tiles last and separately
+        allTiles = [x for x in allTiles if not x.isBonus()]
         for tile in allTiles:
             assert isinstance(tile, Tile)
             if not tile.element in oldTiles.keys():
@@ -376,6 +385,9 @@ class HandBoard(Board):
                 oldTiles[match.element].remove(match)
                 if not len(oldTiles[match.element]):
                     del oldTiles[match.element]
+        oldBoni = dict((x.element, x) for x in self.player.bonusTiles)
+        for newBonusPosition in self.newBonusPositions(newPositions):
+            result[oldBoni[newBonusPosition.element]] = newBonusPosition
         if result:
             self.__avoidCrossingMovements(result)
         return result
@@ -385,19 +397,20 @@ class HandBoard(Board):
         than one identical tile they often switch places - this should not happen.
         So for each element, we make sure that the left-right order is still the
         same as before. For this check, ignore all new tiles"""
-        for yoffset in 0, self.lowerY:
-            items = [x for x in places.items() \
-                     if (x[0].board == self) \
-                        and x[0].yoffset == yoffset \
-                        and x[1] and x[1].yoffset == yoffset \
-                        and not x[0].isBonus()]
-            for element in set(x[0].element for x in items):
-                items = [x for x in places.items() if x[0].element == element]
-                if len(items) > 1:
-                    oldList = sorted(list(x[0] for x in items), key=lambda x:bool(x.board!=self)*1000+x.xoffset)
-                    newList = sorted(list(x[1] for x in items), key=lambda x:x.xoffset)
-                    for idx, oldTile in enumerate(oldList):
-                        places[oldTile] = newList[idx]
+        for yOld in 0, self.lowerY:
+            for yNew in 0, self.lowerY:
+                items = [x for x in places.items() \
+                         if (x[0].board == self) \
+                            and x[0].yoffset == yOld \
+                            and x[1] and x[1].yoffset == yNew \
+                            and not x[0].isBonus()]
+                for element in set(x[0].element for x in items):
+                    items = [x for x in places.items() if x[0].element == element]
+                    if len(items) > 1:
+                        oldList = sorted(list(x[0] for x in items), key=lambda x:bool(x.board!=self)*1000+x.xoffset)
+                        newList = sorted(list(x[1] for x in items), key=lambda x:x.xoffset)
+                        for idx, oldTile in enumerate(oldList):
+                            places[oldTile] = newList[idx]
 
     def __sortPlayerMelds(self):
         """sort player meld lists by their screen position"""
