@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2010 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2010-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 from util import m18nc, m18ncE, logWarning, logException, logDebug
@@ -157,7 +157,7 @@ class MessageViolatesOriginalCall(NotifyAtOnceMessage):
         """the server tells all others"""
         if self.isActivePlayer(table, msg):
             msg.player.mayWin = False
-            table.tellAll(msg.player, Message.ViolatedOriginalCall, table.moved)
+            table.tellAll(msg.player, Message.ViolatedOriginalCall, table.notMoved)
 
 class MessageDiscard(MessageFromClient):
     """the client tells the server whilch tile he discarded"""
@@ -250,7 +250,7 @@ class MessagePickedTile(MessageFromServer):
     """the game server tells us who picked a tile"""
     def clientAction(self, client, move):
         """mirror the picked tile"""
-        assert client.game.pickedTile(move.player, move.deadEnd, tileName=move.source).element == move.source,  \
+        assert client.game.pickedTile(move.player, move.deadEnd, tileName=move.source).element == move.source, \
             (move.player.lastTile, move.source)
         if client.thatWasMe(move.player):
             if move.source[0] in 'fy':
@@ -298,13 +298,15 @@ class MessageMadeOriginalCall(MessageFromServer):
         if client.thatWasMe(move.player):
             answers = [Message.Discard, Message.MahJongg]
             client.ask(move, answers)
+        else:
+            client.ask(move, [Message.OK])
 
 class MessageViolatedOriginalCall(MessageFromServer):
     """the game server tells us who violated an original call"""
     def clientAction(self, client, move):
         """violation: player may not say mah jongg"""
         move.player.mayWin = False
-        if client.thatWasMe(move.player):
+        if not client.thatWasMe(move.player):
             client.ask(move, [Message.OK])
 
 class MessageVoiceId(MessageFromServer):
@@ -449,4 +451,4 @@ def __scanSelf():
                         msg = glob()
                         type.__setattr__(Message, msg.name.replace(' ', ''), msg)
 
-__scanSelf()
\ No newline at end of file
+__scanSelf()
