@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Copyright (C) 2009,2010 Wolfgang Rohdewald <wolfgang@rohdewald.de>
+Copyright (C) 2009-2011 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
 kajongg is free software you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -15,20 +15,19 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 """
 
 import datetime
 
-from PyKDE4.kdeui import KIcon, KDialogButtonBox
-
-from PyQt4.QtCore import SIGNAL, SLOT, Qt, QVariant,  \
+from kde import KIcon, KDialogButtonBox
+from PyQt4.QtCore import SIGNAL, SLOT, Qt, QVariant, \
         QAbstractTableModel
 from PyQt4.QtGui import QDialog, QDialogButtonBox, QWidget, \
-        QHBoxLayout, QVBoxLayout, QAbstractItemView,  \
+        QHBoxLayout, QVBoxLayout, QAbstractItemView, \
         QItemSelectionModel, QGridLayout, QColor, QPalette
 
-from PyKDE4.kdeui import KApplication
+from kde import KApplication
 
 from genericdelegates import RichTextColumnDelegate
 
@@ -225,20 +224,21 @@ class TableList(QWidget):
             return
         assert not InternalParameters.autoPlay
         if self.client.hasLocalServer():
-            title = m18n('Local Games')
+            title = m18n('Local Games with Ruleset %1', self.client.ruleset.name)
         else:
             title = m18n('Tables at %1', self.client.host)
         self.setWindowTitle(title + ' - Kajongg')
         self.view.hideColumn(1)
         tableCount = self.view.model().rowCount(None) if self.view.model() else 0
+        self.view.showColumn(0)
+        self.view.showColumn(2)
+        self.view.showColumn(4)
         if tableCount or not self.client.hasLocalServer():
+            QWidget.show(self)
             if self.client.hasLocalServer():
                 self.view.hideColumn(0)
                 self.view.hideColumn(2)
-            else:
-                self.view.showColumn(0)
-                self.view.showColumn(2)
-            QWidget.show(self)
+                self.view.hideColumn(4)
 
     def afterLogin(self):
         """callback after the server answered our login request"""
@@ -270,14 +270,14 @@ class TableList(QWidget):
 
     def selectTable(self, idx):
         """select table by idx"""
-        self.view.selectionModel().setCurrentIndex(self.view.model().index(idx, 0), QItemSelectionModel.ClearAndSelect)
+        self.view.selectRow(idx)
         self.updateButtonsForTable(self.selectedTable())
 
     def updateButtonsForTable(self, table):
         """update button status for the currently selected table"""
         hasTable = bool(table)
         suspended = hasTable and table.status.startswith('Suspended')
-        suspendedLocalGame = suspended and table.gameid  and self.client.hasLocalServer()
+        suspendedLocalGame = suspended and table.gameid and self.client.hasLocalServer()
         self.joinButton.setEnabled(hasTable and \
             (self.client.username in table.playerNames) == suspended)
         for btn in [self.leaveButton, self.startButton, self.compareButton]:
@@ -302,10 +302,14 @@ class TableList(QWidget):
 
     def newTable(self):
         """I am a slot"""
-        selectDialog = SelectRuleset(self.client.host)
-        if not selectDialog.exec_():
-            return
-        deferred = self.client.callServer('newTable', selectDialog.cbRuleset.current.toList(),
+        if self.client.hasLocalServer():
+            ruleset = self.client.ruleset
+        else:
+            selectDialog = SelectRuleset(self.client.host)
+            if not selectDialog.exec_():
+                return
+            ruleset = selectDialog.cbRuleset.current
+        deferred = self.client.callServer('newTable', ruleset.toList(),
             InternalParameters.playOpen, InternalParameters.autoPlay, InternalParameters.seed)
         if self.client.hasLocalServer():
             self.hideForever = True
@@ -314,13 +318,19 @@ class TableList(QWidget):
     def gotTables(self, tables):
         """got tables for first time. If we play a local game and we have no
         suspended game, automatically start a new one"""
-        if InternalParameters.autoPlay or (not tables and self.client.hasLocalServer()):
+        if not InternalParameters.autoPlay:
+            clientTables = list(ClientTable(*x) for x in tables)  # pylint: disable=W0142
+            if self.client.hasLocalServer():
+                # when playing a local game, only show pending tables with
+                # previously selected ruleset
+                clientTables = list(x for x in clientTables if x.ruleset == self.client.ruleset)
+        if InternalParameters.autoPlay or (not clientTables and self.client.hasLocalServer()):
             self.hideForever = True
             self.client.callServer('newTable', self.client.ruleset.toList(), InternalParameters.playOpen,
                 InternalParameters.autoPlay,
                 InternalParameters.seed).addCallback(self.newLocalTable)
         else:
-            self.load([ClientTable(*x) for x in tables]) # pylint: disable=W0142
+            self.loadTables(clientTables)
             self.show()
 
     def selectedTable(self):
@@ -359,7 +369,7 @@ class TableList(QWidget):
         """leave a table"""
         self.client.callServer('leaveTable', self.selectedTable().tableid)
 
-    def load(self, tables):
+    def loadTables(self, tables):
         """build and use a model around the tables.
         Show all new tables (no gameid given yet) and all suspended
         tables that also exist locally. In theory all suspended games should
@@ -376,12 +386,12 @@ class TableList(QWidget):
         self.connect(selection,
             SIGNAL("selectionChanged ( QItemSelection, QItemSelection)"),
             self.selectionChanged)
-        self.selectTable(0)
         if len(tables) == 1:
             self.startButton.setFocus()
         elif not tables:
             self.newButton.setFocus()
-        if not self.selectedTable() and self.view.model().rowCount():
-            self.selectTable(0)
         if not InternalParameters.autoPlay:
             self.show()
+        if not self.selectedTable() and self.view.model().rowCount():
+            self.selectTable(0)
+        self.view.setFocus()
