@@ -19,8 +19,8 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
-#ifndef _SCOREDLG_H_
-#define _SCOREDLG_H_
+#ifndef KONQUEST_SCOREDLG_H
+#define KONQUEST_SCOREDLG_H
 
 
 #include <QTableWidget>
@@ -45,5 +45,4 @@ private:
 
 };
 
-#endif // _SCOREDLG_H_
-
+#endif // KONQUEST_SCOREDLG_H
