@@ -181,6 +181,9 @@ public:
     bool dropQueued;
     bool newCardsQueued;
 
+    QList<QPair<KCard*,KCardPile*> > multiStepMoves;
+    int multiStepDuration;
+
     QStack<GameState*> undoStack;
     GameState * currentState;
     QStack<GameState*> redoStack;
@@ -279,9 +282,10 @@ void DealerScene::openGame(QDomDocument &doc)
                     QDomElement card = cardNodes.item(j).toElement();
                     int s = card.attribute("suit").toInt();
                     int r = card.attribute("value").toInt();
-
+                    
                     KCard * c = cards.take( ( s << 4 ) + r );
-                    Q_ASSERT( c );
+                    if (!c)
+                        continue;
 
                     c->setFaceUp(card.attribute("faceup").toInt());
                     p->add(c);
@@ -289,7 +293,7 @@ void DealerScene::openGame(QDomDocument &doc)
                 break;
             }
         }
-        p->layoutCards( 0 );
+        updatePileLayout( p, 0 );
     }
     setGameState( dealer.attribute("data") );
 
@@ -418,20 +422,22 @@ QList<PatPile*> DealerScene::patPiles() const
 }
 
 
-void DealerScene::moveCardsToPile( QList<KCard*> cards, KCardPile * pile, int duration )
+void DealerScene::cardsMoved( const QList<KCard*> & cards, KCardPile * oldPile, KCardPile * newPile )
 {
-    PatPile * newPile = dynamic_cast<PatPile*>( pile );
-    foreach ( KCard * c, cards )
+    PatPile * newPatPile = dynamic_cast<PatPile*>( newPile );
+    PatPile * oldPatPile = dynamic_cast<PatPile*>( oldPile );
+
+    if ( oldPatPile && oldPatPile->isFoundation() && newPatPile && !newPatPile->isFoundation() )
     {
-        PatPile * oldPile = dynamic_cast<PatPile*>( c->pile() );
-        if ( oldPile && oldPile->isFoundation() && !newPile->isFoundation() )
+        foreach ( KCard * c, cards )
             d->cardsNotToDrop.insert( c );
-        else
+    }
+    else
+    {
+        foreach ( KCard * c, cards )
             d->cardsNotToDrop.remove( c );
     }
 
-    KCardScene::moveCardsToPile( cards, pile, duration );
-
     if ( !d->dropInProgress && !d->dealInProgress )
     {
         d->gameStarted = true;
@@ -488,7 +494,6 @@ bool DealerScene::isHintActive() const
     return d->hintInProgress;
 }
 
-
 QList<MoveHint> DealerScene::getSolverHints()
 {
     QList<MoveHint> hintList;
@@ -497,23 +502,16 @@ QList<MoveHint> DealerScene::getSolverHints()
         d->solverThread->abort();
 
     solver()->translate_layout();
-    solver()->patsolve( 1 );
+    bool debug = false;
+#if DEBUG_HINTS
+    debug = true;
+#endif
+    solver()->patsolve( 1, debug );
 
     foreach ( const MOVE & m, solver()->firstMoves )
     {
-
-#if DEBUG_HINTS
-        if ( m.totype == O_Type )
-            fprintf( stderr, "   move from %d out (at %d) Prio: %d\n", m.from,
-                     m.turn_index, m.pri );
-        else
-            fprintf( stderr, "   move from %d to %d (%d) Prio: %d\n", m.from, m.to,
-                     m.turn_index, m.pri );
-#endif
-
         MoveHint mh = solver()->translateMove( m );
-        if ( mh.isValid() )
-            hintList << mh;
+	hintList << mh;
     }
     return hintList;
 }
@@ -575,6 +573,11 @@ QList<MoveHint> DealerScene::getHints()
     return hintList;
 }
 
+static bool prioSort(const MoveHint &c1, const MoveHint &c2) 
+{
+  return c1.priority() < c2.priority();
+}
+
 
 MoveHint DealerScene::chooseHint()
 {
@@ -607,7 +610,7 @@ MoveHint DealerScene::chooseHint()
         qreal randomExp = qMin<qreal>( -log( 1 - qreal( KRandom::random() ) / RAND_MAX ) / 4, 1 );
         int randomIndex =  randomExp * ( hintList.size() - 1 );
 
-        qSort( hintList.begin(), hintList.end(), qGreater<MoveHint>() );
+	qSort(hintList.begin(), hintList.end(), prioSort);
         return hintList.at( randomIndex );
     }
 }
@@ -794,7 +797,7 @@ void DealerScene::updateWonItem()
 
 bool DealerScene::allowedToAdd( const KCardPile * pile, const QList<KCard*> & cards ) const
 {
-    if ( !pile->isEmpty() && !pile->top()->isFaceUp() )
+    if ( !pile->isEmpty() && !pile->topCard()->isFaceUp() )
         return false;
 
     const PatPile * p = dynamic_cast<const PatPile*>( pile );
@@ -852,7 +855,7 @@ void DealerScene::mousePressEvent( QGraphicsSceneMouseEvent * e )
     else if ( e->button() == Qt::RightButton
               && card
               && card->pile()
-              && card != card->pile()->top()
+              && card != card->pile()->topCard()
               && cardsBeingDragged().isEmpty()
               && !isCardAnimationRunning() )
     {
@@ -878,7 +881,7 @@ void DealerScene::mouseReleaseEvent( QGraphicsSceneMouseEvent * e )
     if ( e->button() == Qt::RightButton && d->peekedCard && d->peekedCard->pile() )
     {
         e->accept();
-        d->peekedCard->pile()->layoutCards( DURATION_FANCYSHOW );
+        updatePileLayout( d->peekedCard->pile(), DURATION_FANCYSHOW );
         d->peekedCard = 0;
     }
     else
@@ -904,7 +907,7 @@ bool DealerScene::tryAutomaticMove( KCard * card )
     if ( !isCardAnimationRunning()
          && card
          && card->pile()
-         && card == card->pile()->top()
+         && card == card->pile()->topCard()
          && card->isFaceUp()
          && allowedToRemove( card->pile(), card ) )
     {
@@ -1113,7 +1116,7 @@ void DealerScene::setState( GameState * state )
         foreach ( KCard * c, p->cards() )
             p->swapCards( p->indexOf( c ), cardIndices.value( c ) );
 
-        p->layoutCards(0);
+        updatePileLayout( p, 0 );
     }
 
     // restore game-specific information
@@ -1317,6 +1320,12 @@ void DealerScene::animationDone()
 {
     Q_ASSERT( !isCardAnimationRunning() );
 
+    if ( !d->multiStepMoves.isEmpty() )
+    {
+        continueMultiStepMove();
+        return;
+    }
+
     if ( d->takeStateQueued )
     {
         d->takeStateQueued = false;
@@ -1408,11 +1417,11 @@ void DealerScene::demo()
     MoveHint mh = chooseHint();
     if ( mh.isValid() )
     {
-        kDebug(mh.pile()->top()) << "Moving" << mh.card()->objectName()
+        kDebug(mh.pile()->topCard()) << "Moving" << mh.card()->objectName()
                                  << "from the" << mh.card()->pile()->objectName()
                                  << "pile to the" << mh.pile()->objectName()
-                                 << "pile, putting it on top of" << mh.pile()->top()->objectName();
-        kDebug(!mh.pile()->top()) << "Moving" << mh.card()->objectName()
+                                 << "pile, putting it on top of" << mh.pile()->topCard()->objectName();
+        kDebug(!mh.pile()->topCard()) << "Moving" << mh.card()->objectName()
                                   << "from the" << mh.card()->pile()->objectName()
                                   << "pile to the" << mh.pile()->objectName()
                                   << "pile, which is empty";
@@ -1687,7 +1696,11 @@ QImage DealerScene::createDump() const
     return img2;
 }
 
-void DealerScene::mapOldId(int) {}
+
+void DealerScene::mapOldId( int id )
+{
+    Q_UNUSED( id );
+}
 
 
 int DealerScene::oldId() const
@@ -1696,6 +1709,30 @@ int DealerScene::oldId() const
 }
 
 
+QString DealerScene::getGameState() const
+{
+    return QString();
+}
+
+
+void DealerScene::setGameState( const QString & state )
+{
+    Q_UNUSED( state );
+}
+
+
+QString DealerScene::getGameOptions() const
+{
+    return QString();
+}
+
+
+void DealerScene::setGameOptions( const QString & options )
+{
+    Q_UNUSED( options );
+}
+
+
 bool DealerScene::allowedToStartNewGame()
 {
     // Check if the user is already running a game, and if she is,
@@ -1730,7 +1767,7 @@ void DealerScene::startDealAnimation()
     qreal speed = sqrt( width() * width() + height() * height() ) / ( DURATION_DEAL );
     foreach ( PatPile * p, patPiles() )
     {
-        p->layoutCards(0);
+        updatePileLayout( p, 0 );
         foreach ( KCard * c, p->cards() )
         {
             if ( !d->initDealPositions.contains( c ) )
@@ -1749,5 +1786,127 @@ void DealerScene::startDealAnimation()
 }
 
 
+void DealerScene::multiStepMove( const QList<KCard*> & cards,
+                                 KCardPile * pile,
+                                 const QList<KCardPile*> & freePiles,
+                                 const QList<KCardPile*> & freeCells,
+                                 int duration )
+{
+    Q_ASSERT( cards.size() == 1 || !freePiles.isEmpty() || !freeCells.isEmpty() );
+
+    d->multiStepMoves.clear();
+    d->multiStepDuration = duration;
+
+    multiStepSubMove( cards, pile, freePiles, freeCells );
+    continueMultiStepMove();
+}
+
+
+void DealerScene::multiStepSubMove( QList<KCard*> cards,
+                                    KCardPile * pile,
+                                    QList<KCardPile*> freePiles,
+                                    const QList<KCardPile*> & freeCells )
+{
+    // Note that cards and freePiles are passed by value, as we need to make a
+    // local copy anyway.
+
+    // Using n free cells, we can move a run of n+1 cards. If we want to move
+    // more than that, we have to recursively move some of our cards to one of
+    // the free piles temporarily.
+    const int freeCellsPlusOne = freeCells.size() + 1;
+    int cardsToSubMove = cards.size() - freeCellsPlusOne;
+
+    QList<QPair<KCardPile*,QList<KCard*> > > tempMoves;
+    while ( cardsToSubMove > 0 )
+    {
+        int tempMoveSize;
+        if ( cardsToSubMove <= freePiles.size() * freeCellsPlusOne )
+        {
+            // If the cards that have to be submoved can be spread across the
+            // the free piles without putting more than freeCellsPlusOne cards
+            // on each one, we do so. This means that none of our submoves will
+            // need further submoves, which keeps the total move count down. We
+            // Just to a simple rounding up integer division.
+            tempMoveSize = (cardsToSubMove + freePiles.size() - 1) / freePiles.size();
+        }
+        else
+        {
+            // Otherwise, we use the space optimal method that gets the cards
+            // moved using a minimal number of piles, but uses more submoves.
+            tempMoveSize = freeCellsPlusOne;
+            while ( tempMoveSize * 2 < cardsToSubMove )
+                tempMoveSize *= 2;
+        }
+
+        QList<KCard*> subCards;
+        for ( int i = 0; i < tempMoveSize; ++i )
+            subCards.prepend( cards.takeLast() );
+
+        Q_ASSERT( !freePiles.isEmpty() );
+        KCardPile * nextPile = freePiles.takeFirst();
+
+        tempMoves << qMakePair( nextPile, subCards );
+        multiStepSubMove( subCards, nextPile, freePiles, freeCells );
+
+        cardsToSubMove -= tempMoveSize;
+    }
+
+    // Move cards to free cells.
+    for ( int i = 0; i < cards.size() - 1; ++i )
+    {
+        KCard * c = cards.at( cards.size() - 1 - i );
+        d->multiStepMoves << qMakePair( c, freeCells[i] );
+    }
+
+    // Move bottom card to destination pile.
+    d->multiStepMoves << qMakePair( cards.first(), pile );
+
+    // Move cards from free cells to destination pile.
+    for ( int i = 1; i < cards.size(); ++i )
+        d->multiStepMoves << qMakePair( cards.at( i ), pile );
+
+    // If we just moved the bottomost card of the source pile, it must now be
+    // empty and we won't need it any more. So we return it to the list of free
+    // piles.
+    KCardPile * sourcePile = cards.first()->pile();
+    if ( sourcePile->at( 0 ) == cards.first() )
+        freePiles << sourcePile;
+
+    // If we had to do any submoves, we now move those cards from their
+    // temporary pile to the destination pile and free up their temporary pile.
+    while ( !tempMoves.isEmpty() )
+    {
+        QPair<KCardPile*, QList<KCard*> > m = tempMoves.takeLast();
+        multiStepSubMove( m.second, pile, freePiles, freeCells );
+        freePiles << m.first;
+    }
+}
+
+
+void DealerScene::continueMultiStepMove()
+{
+    Q_ASSERT( !d->multiStepMoves.isEmpty() );
+    Q_ASSERT( !isCardAnimationRunning() );
+
+    QPair<KCard*,KCardPile*> m = d->multiStepMoves.takeFirst();
+    KCard * card = m.first;
+    KCardPile * dest = m.second;
+    KCardPile * source = card->pile();
+
+    Q_ASSERT( card == source->topCard() );
+    Q_ASSERT( allowedToAdd( dest, QList<KCard*>() << card ) );
+
+    d->multiStepDuration = qMax<int>( d->multiStepDuration * 0.9, 50 );
+
+    dest->add( card );
+    card->raise();
+    updatePileLayout( dest, d->multiStepDuration );
+    updatePileLayout( source, d->multiStepDuration );
+
+    if ( d->multiStepMoves.isEmpty() )
+        takeState();
+}
+
+
 #include "dealer.moc"
 #include "moc_dealer.cpp"
