@@ -48,7 +48,7 @@ public:
 
 
 protected:
-    virtual void setGameState(const QString &);
+    virtual void setGameState( const QString & state );
     virtual bool checkAdd(const PatPile * pile, const QList<KCard*> & oldCards, const QList<KCard*> & newCards) const;
     virtual bool checkRemove(const PatPile * pile, const QList<KCard*> & cards) const;
     virtual void restart( const QList<KCard*> & cards );
