@@ -37,6 +37,8 @@
 
 namespace
 {
+    const QString cacheNameTemplate( "libkcardgame-themes/%1" );
+    const QString timeStampKey( "libkcardgame_timestamp" );
     const QString unscaledSizeKey( "libkcardgame_unscaledsize" );
     const QString lastUsedSizeKey( "libkcardgame_lastusedsize" );
 
@@ -87,18 +89,8 @@ void RenderingThread::run()
         if ( !d->cache->contains( key ) )
         {
             kDebug() << "Renderering" << key << "in rendering thread.";
-
-            QImage img = QImage( m_size, QImage::Format_ARGB32 );
-            img.fill( Qt::transparent );
-            QPainter p( &img );
-            {
-                QMutexLocker l( &(d->rendererMutex) );
-                d->renderer()->render( &p, element );
-            }
-            p.end();
-
+            QImage img = d->renderCard( element, m_size );
             d->cache->insertImage( key, img );
-
             emit renderingDone( element, img );
         }
     }
@@ -142,17 +134,14 @@ QSvgRenderer * KAbstractCardDeckPrivate::renderer()
 }
 
 
-QPixmap KAbstractCardDeckPrivate::renderCard( const QString & element )
+QImage KAbstractCardDeckPrivate::renderCard( const QString & element, const QSize & size )
 {
-    QString key = keyForPixmap( element , currentCardSize );
-    kDebug() << "Renderering" << key << "in main thread.";
-
     // Note that we don't use Format_ARGB32_Premultiplied as it sacrifices some
     // colour accuracy at low opacities for performance. Normally this wouldn't
     // be an issue, but in card games we often will have, say, 52 pixmaps
     // stacked on top of one another, which causes these colour inaccuracies to
     // add up to the point that they're very visible.
-    QImage img( currentCardSize, QImage::Format_ARGB32 );
+    QImage img( size, QImage::Format_ARGB32 );
     img.fill( Qt::transparent );
     QPainter p( &img );
     {
@@ -173,9 +162,7 @@ QPixmap KAbstractCardDeckPrivate::renderCard( const QString & element )
     }
     p.end();
 
-    cache->insertImage( key, img );
-
-    return QPixmap::fromImage( img );
+    return img;
 }
 
 
@@ -214,12 +201,20 @@ QPixmap KAbstractCardDeckPrivate::requestPixmap( quint32 id, bool faceUp )
     QPixmap & stored = it.value().cardPixmap;
     if ( stored.size() != currentCardSize )
     {
-        if ( !cache->findPixmap( keyForPixmap( elementId , currentCardSize ), &stored ) )
+        QString key = keyForPixmap( elementId , currentCardSize );
+        if ( !cache->findPixmap( key, &stored ) )
         {
             if ( stored.isNull() )
-                stored = renderCard( elementId );
+            {
+                kDebug() << "Renderering" << key << "in main thread.";
+                QImage img = renderCard( elementId, currentCardSize );
+                cache->insertImage( key, img );
+                stored = QPixmap::fromImage( img );
+            }
             else
+            {
                 stored = stored.scaled( currentCardSize );
+            }
         }
         Q_ASSERT( stored.size() == currentCardSize );
     }
@@ -459,7 +454,22 @@ void KAbstractCardDeck::setTheme( const KCardTheme & theme )
         }
 
         delete d->cache;
-        d->cache = createCache( d->theme );
+
+        QString cacheName = QString( cacheNameTemplate ).arg( theme.dirName() );
+        d->cache = new KImageCache( cacheName, 3 * 1024 * 1024 );
+        d->cache->setEvictionPolicy( KSharedDataCache::EvictLeastRecentlyUsed );
+
+        // Enabling the pixmap cache has caused issues: we were getting back
+        // different pixmaps than we had inserted. We keep a partial cache of the
+        // pixmaps in KAbstractCardDeck already, so the builtin pixmap caching
+        // doesn't really add that much benefit anyway.
+        d->cache->setPixmapCaching( false );
+
+        if ( d->cache->timestamp() < theme.lastModified().toTime_t() )
+        {
+            d->cache->clear();
+            d->cache->setTimestamp( theme.lastModified().toTime_t() );
+        }
 
         d->originalCardSize = d->unscaledCardSize();
         Q_ASSERT( !d->originalCardSize.isNull() );
@@ -484,6 +494,12 @@ bool KAbstractCardDeck::hasAnimatedCards() const
     return !d->cardsWaitedFor.isEmpty();
 }
 
+void KAbstractCardDeck::stopAnimations() 
+{
+    foreach ( KCard * c, d->cardsWaitedFor )
+       c->stopAnimation();
+    d->cardsWaitedFor.clear();
+}
 
 QPixmap KAbstractCardDeck::cardPixmap( quint32 id, bool faceUp )
 {
