@@ -1,5 +1,5 @@
 /*
- *  Copyright 2007-2009  Parker Coates <parker.coates@kdemail.net>
+ *  Copyright 2007-2010  Parker Coates <parker.coates@kdemail.net>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
@@ -19,42 +19,28 @@
 #ifndef RENDERER_H
 #define RENDERER_H
 
-#include <KGameTheme>
-
-#include <KPixmapCache>
+#include <KGameRenderer>
 
 class QSize;
 class QString;
 #include <QtCore/Qt>
 class QColor;
 class QPixmap;
-#include <QtSvg/QSvgRenderer>
 
 
-class Renderer
+class Renderer : public KGameRenderer
 {
 public:
     static Renderer * self();
 
-    bool setTheme( const QString & fileName );
-    const KGameTheme & theme() const;
-
-    QPixmap renderElement( const QString & elementId, QSize size );
-    QSize sizeOfElement( const QString & elementId );
     qreal aspectRatioOfElement( const QString & elementId );
     QColor colorOfElement( const QString & elementId );
-    QPixmap renderGamePreview( int id, QSize size );
 
 private:
     Renderer();
 
-    void ensureSvgIsLoaded();
-
-    QSvgRenderer m_svgRenderer;
-    KPixmapCache m_pixmapCache;
-    QHash<QString, QColor> m_colors;
-    KGameTheme m_theme;
-    bool m_themeIsLoaded;
+    QHash<QString,QColor> m_colors;
+    QString m_cachedTheme;
 
     friend class RendererPrivate;
 };
