@@ -39,6 +39,8 @@
 #include "soundfactory.h"
 #include "playgrounddelegate.h"
 
+static const char *DEFAULT_THEME = "default_theme.theme";
+
 // Constructor
 TopLevel::TopLevel()
   : KXmlGuiWindow(0)
@@ -126,8 +128,6 @@ void TopLevel::changeGameboard()
 void TopLevel::changeGameboard(const QString &newGameBoard)
 {
   if (newGameBoard == playGround->currentGameboard()) return;
-  int index = playgroundCombo->findData(newGameBoard,BOARD_THEME);
-  playgroundCombo->setCurrentIndex(index);
 
   QString fileToLoad;
   QFileInfo fi(newGameBoard);
@@ -141,16 +141,27 @@ void TopLevel::changeGameboard(const QString &newGameBoard)
     fileToLoad = newGameBoard;
   }
 
-  if (playGround->loadPlayGround(fileToLoad))
+  int index = playgroundCombo->findData(fileToLoad, BOARD_THEME);
+  playgroundCombo->setCurrentIndex(index);
+  QAction *action = actionCollection()->action(fileToLoad);
+  if (action && playGround->loadPlayGround(fileToLoad))
   {
-    actionCollection()->action(fileToLoad)->setChecked(true);
+    action->setChecked(true);
 
     // Change gameboard in the remembered options
     writeOptions();
   }
   else
   {
-    KMessageBox::error(this, i18n("Error while loading the playground."));
+    // Something bad just happened, try the default playground
+    if (newGameBoard != QLatin1String(DEFAULT_THEME))
+    {
+      changeGameboard(QLatin1String(DEFAULT_THEME));
+    }
+    else
+    {
+      KMessageBox::error(this, i18n("Error while loading the playground."));
+    }
   }
 }
 
@@ -179,9 +190,10 @@ void TopLevel::changeLanguage(const QString &soundFile)
   }
 
   // Change language effectively
-  if (soundFactory->loadLanguage(fileToLoad))
+  QAction *action = actionCollection()->action(fileToLoad);
+  if (action && soundFactory->loadLanguage(fileToLoad))
   {
-    actionCollection()->action(fileToLoad)->setChecked(true);
+    action->setChecked(true);
 
     // Change language in the remembered options
     writeOptions();
@@ -206,7 +218,7 @@ void TopLevel::readOptions(QString &board, QString &language)
 
   QString option = config.readEntry("Sound",  "on" );
   bool soundEnabled = option.indexOf(QLatin1String( "on" )) == 0;
-  board = config.readEntry("Gameboard", "default_theme.theme");
+  board = config.readEntry("Gameboard", DEFAULT_THEME);
   language = config.readEntry("Language", "" );
   bool keepAspectRatio = config.readEntry("KeepAspectRatio", false);
 
@@ -287,7 +299,7 @@ void TopLevel::setupKAction()
 
   connect(playgroundCombo, SIGNAL(currentIndexChanged(int)),this,SLOT(changeGameboardFromCombo(int)));
 
-  QWidgetAction *widgetAction = new QWidgetAction(this);
+  KAction *widgetAction = new KAction(this);
   widgetAction->setDefaultWidget(playgroundCombo);
   actionCollection()->addAction( QLatin1String( "playgroundSelection" ),widgetAction);
 
