@@ -22,8 +22,8 @@
 #include <KLocale>
 #include <KStandardDirs>
 #include <KConfigSkeleton>
-#include <knewstuff2/engine.h>
 #include <KComponentData>
+#include <KNS3/DownloadDialog>
 
 #include "ui_kgamethemeselector.h"
 #include "kgametheme.h"
@@ -42,10 +42,11 @@ class KGameThemeSelector::KGameThemeSelectorPrivate
         QString groupName;
 
         void setupData(KConfigSkeleton* config, KGameThemeSelector::NewStuffState knsflags);
-        void findThemes(QString &initialGroup);
+        void findThemes(const QString &initialSelection);
 
         // private slots
         void _k_updatePreview();
+        void _k_updateThemeList(const QString& strTheme);
         void _k_openKNewStuffDialog();
 };
 
@@ -65,78 +66,106 @@ KGameThemeSelector::~KGameThemeSelector()
 void KGameThemeSelector::KGameThemeSelectorPrivate::setupData(KConfigSkeleton * aconfig, KGameThemeSelector::NewStuffState knsflags)
 {
     ui.setupUi(q);
-    ui.getNewButton->setIcon(KIcon("get-hot-new-stuff")); 
-    //Get our currently configured Tileset entry
-    KConfig * config = aconfig->config();
-    KConfigGroup group = config->group("General");
-    QString initialGroup = group.readEntry("Theme"); //Should be someting like "themes/default.desktop"
+    ui.getNewButton->setIcon(KIcon( QLatin1String( "get-hot-new-stuff" )));
 
-    //The lineEdit widget holds our bg path, but the user does not manipulate it directly
+    //The lineEdit widget holds our theme path for automatic connection via KConfigXT.
+    //But the user should not manipulate it directly, so we hide it.
     ui.kcfg_Theme->hide();
-    
+    connect(ui.kcfg_Theme, SIGNAL(textChanged(const QString&)), q, SLOT(_k_updateThemeList(const QString&)));
+
     //Disable KNS button?
     if (knsflags==KGameThemeSelector::NewStuffDisableDownload) {
       ui.getNewButton->hide();
     }
 
-    //Now get our tilesets into a list
-    KGlobal::dirs()->addResourceType("gamethemeselector", "data", KGlobal::mainComponent().componentName() + '/' + lookupDirectory + '/');
-    findThemes(initialGroup);
+    //Get the last used theme path from the KConfigSkeleton
+    KConfigSkeletonItem * configItem = aconfig->findItem(QLatin1String( "Theme" ));
+    QString lastUsedTheme = configItem->property().toString();
+
+    //Now get our themes into the list widget
+    KGlobal::dirs()->addResourceType("gamethemeselector", "data", KGlobal::mainComponent().componentName() + QLatin1Char( '/' ) + lookupDirectory + QLatin1Char( '/' ));
+    findThemes(lastUsedTheme);
+
     connect(ui.getNewButton, SIGNAL(clicked()), q, SLOT(_k_openKNewStuffDialog()));
 }
 
-void KGameThemeSelector::KGameThemeSelectorPrivate::findThemes(QString &initialGroup)
+void KGameThemeSelector::KGameThemeSelectorPrivate::findThemes(const QString &initialSelection)
 {
-    
-    QStringList themesAvailable;
-    KGlobal::dirs()->findAllResources("gamethemeselector", QString("*.desktop"), KStandardDirs::Recursive, themesAvailable);
-    QString namestr("Name");
-    int numvalidentries = 0;
     qDeleteAll(themeMap.values());
     themeMap.clear();
+
     //Disconnect the themeList as we are going to clear it and do not want previews generated
     ui.themeList->disconnect();
     ui.themeList->clear();
-    
-    for (int i = 0; i < themesAvailable.size(); ++i)
+    ui.themeList->setSortingEnabled(true);
+
+    QStringList themesAvailable;
+    KGlobal::dirs()->findAllResources("gamethemeselector", QLatin1String( "*.desktop" ), KStandardDirs::Recursive, themesAvailable);
+
+    bool initialFound = false;
+    foreach (const QString &file, themesAvailable)
     {
+      QString themePath = lookupDirectory + QLatin1Char( '/' ) + file;
       KGameTheme* atheme = new KGameTheme(groupName);
-      QString themepath = lookupDirectory + '/' + themesAvailable.at(i);
-      if (atheme->load(themepath)) {
-        themeMap.insert(atheme->themeProperty(namestr), atheme);
-        ui.themeList->addItem(atheme->themeProperty(namestr));
-        //Find if this is our currently configured Theme
-        if (themepath==initialGroup) {
-                    //Select current entry
-          ui.themeList->setCurrentRow(numvalidentries);
+
+      if (atheme->load(themePath)) {
+        QString themeName = atheme->themeProperty(QLatin1String( "Name" ));
+        //Add underscores to avoid duplicate names.
+        while (themeMap.contains(themeName))
+          themeName += QLatin1Char( '_' );
+        themeMap.insert(themeName, atheme);
+        QListWidgetItem * item = new QListWidgetItem(themeName, ui.themeList);
+
+        //Find if this is our currently configured theme
+        if (themePath==initialSelection) {
+          initialFound = true;
+          ui.themeList->setCurrentItem(item);
           _k_updatePreview();
         }
-        ++numvalidentries;
       } else {
         delete atheme;
       }
     }
-    ui.themeList->sortItems();
-    //reconnect the themeList
+
+    if (!initialFound)
+    {
+      // TODO change this if we ever change KGameTheme::loadDefault
+      QString defaultPath = QLatin1String( "themes/default.desktop" );
+      foreach(KGameTheme* theme, themeMap)
+      {
+        if (theme->path().endsWith(defaultPath))
+        {
+          const QList<QListWidgetItem *> itemList = ui.themeList->findItems(theme->themeProperty(QLatin1String( "Name" )), Qt::MatchExactly);
+          // never can be != 1 but better safe than sorry
+          if (itemList.count() == 1)
+          {
+            ui.themeList->setCurrentItem(itemList.first());
+            _k_updatePreview();
+          }
+        }
+      }
+    }
+
+    //Reconnect the themeList
     connect(ui.themeList, SIGNAL(currentItemChanged ( QListWidgetItem * , QListWidgetItem * )), q, SLOT(_k_updatePreview()));
 }
 
 void KGameThemeSelector::KGameThemeSelectorPrivate::_k_updatePreview()
 {
     KGameTheme * seltheme = themeMap.value(ui.themeList->currentItem()->text());
-        //Sanity checkings. Should not happen.
+    //Sanity checkings. Should not happen.
     if (!seltheme) return;
     if (seltheme->path() == ui.kcfg_Theme->text()) {
         return;
     }
     ui.kcfg_Theme->setText(seltheme->fileName());
 
-    QString authstr("Author");
-    QString contactstr("AuthorEmail");
-    QString descstr("Description");
+    QString authstr(QLatin1String( "Author" ));
+    QString contactstr(QLatin1String( "AuthorEmail" ));
+    QString descstr(QLatin1String( "Description" ));
     QString emailstr;
     if (!seltheme->themeProperty(contactstr).isEmpty() ) {
-        emailstr = QString("<a href=\"mailto:%1\">%1</a>").arg(seltheme->themeProperty(contactstr));
+        emailstr = QString::fromLatin1( "<a href=\"mailto:%1\">%1</a>").arg(seltheme->themeProperty(contactstr));
     }
 
     ui.themeAuthor->setText(seltheme->themeProperty(authstr));
@@ -146,17 +175,31 @@ void KGameThemeSelector::KGameThemeSelectorPrivate::_k_updatePreview()
     //Draw the preview
     QPixmap pix(seltheme->preview());
     ui.themePreview->setPixmap(pix.scaled(ui.themePreview->size(),Qt::KeepAspectRatio,Qt::SmoothTransformation));
+}
 
+void KGameThemeSelector::KGameThemeSelectorPrivate::_k_updateThemeList(const QString& strTheme)
+{
+    //find theme and set selection to the current theme; happens when pressing "Default"
+    QListWidgetItem * currentItem = ui.themeList->currentItem();
+    if(!currentItem || themeMap.value(currentItem->text())->fileName() != strTheme)
+    {
+        for(int i = 0; i < ui.themeList->count(); i++)
+        {
+            if(themeMap.value(ui.themeList->item(i)->text())->fileName() == strTheme)
+            {
+                ui.themeList->setCurrentItem(ui.themeList->item(i));
+                break;
+            }
+        }
+    }
 }
 
 void KGameThemeSelector::KGameThemeSelectorPrivate::_k_openKNewStuffDialog()
 {
-    KNS::Entry::List entries = KNS::Engine::download();
-    //rescan themes directory
-    QString currentthemepath = ui.kcfg_Theme->text();
-    if (entries.size()>0) findThemes(currentthemepath);
-    //Needed as the static KNS constructor made copies
-    qDeleteAll(entries);
+    KNS3::DownloadDialog dialog( q );
+    dialog.exec();
+    if ( !dialog.changedEntries().isEmpty() )
+        findThemes( ui.kcfg_Theme->text() );
 }
 
 #include "kgamethemeselector.moc"
