@@ -1,5 +1,6 @@
 /***************************************************************************
  *   Copyright 2010 Stefan Majewsky <majewsky@gmx.net>
+ *   Copyright 2010 Johannes Löhnert <loehnert.kde@gmx.de>
  *
  *   This program is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU General Public
@@ -34,10 +35,16 @@ namespace Palapeli
 		Q_OBJECT
 		Q_PROPERTY(qreal activeShadowOpacity READ activeShadowOpacity WRITE setActiveShadowOpacity)
 		public:
+			///This constructor is used when the piece is loaded only from the puzzle file.
+			explicit Piece(const QImage& pieceImage, const QPoint& offset);
 			///This constructor creates a piece without a shadow, unless a shadow is provided explicitly.
 			explicit Piece(const Palapeli::PieceVisuals& pieceVisuals, const Palapeli::PieceVisuals& shadowVisuals = Palapeli::PieceVisuals());
-			///This method will create a shadow for this piece if there is none ATM.
-			void createShadow();
+			///This method will
+			///\li create a shadow for this piece if there is none ATM.
+			///\li apply the bevel map to the piece pixmap.
+			///These operations need not be done until the piece is about to be shown.
+			///\return whether something needed to be done
+			bool completeVisuals();
 
 			///Returns the bounding rect without shadows.
 			QRectF bareBoundingRect() const;
@@ -76,6 +83,7 @@ namespace Palapeli
 		private Q_SLOTS:
 			void pieceItemSelectedChanged(bool selected);
 		private:
+			void commonInit(const Palapeli::PieceVisuals& pieceVisuals);
 			void createShadowItems(const Palapeli::PieceVisuals& shadowVisuals);
 			qreal activeShadowOpacity() const;
 			void setActiveShadowOpacity(qreal opacity);
